/* Local declarations for termcap library.
   Copyright (C) 1999 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.  */

#ifndef _LTCAP_H_
#define _LTCAP_H_ 1

#if !defined (__APPLE__)
#  define __private_extern__
#endif

#ifndef MAX_TGETENT_BUFSIZ
#  define MAX_TGETENT_BUFSIZ 2048
#endif

#endif /* _LTCAP_H_ */
